@@ -29,6 +29,8 @@ logger = logging.getLogger(__name__)
 TDD_CHOICE_COLUMN = 'tdd'
 USE_BRUTE_FORCE_TO_COMPUTE_LOGSUMS = False
 
+RUN_ALTS_PREPROCESSOR_BEFORE_MERGE = True  # see FIXME below before changing this
+
 
 def skims_for_logsums(tour_purpose, model_settings, trace_label):
 
@@ -39,7 +41,14 @@ def skims_for_logsums(tour_purpose, model_settings, trace_label):
     skim_dict = network_los.get_default_skim_dict()
 
     orig_col_name = 'home_zone_id'
-    dest_col_name = model_settings.get('DESTINATION_FOR_TOUR_PURPOSE').get(tour_purpose)
+
+    destination_for_tour_purpose = model_settings.get('DESTINATION_FOR_TOUR_PURPOSE')
+    if isinstance(destination_for_tour_purpose, str):
+        dest_col_name = destination_for_tour_purpose
+    elif isinstance(destination_for_tour_purpose, dict):
+        dest_col_name = destination_for_tour_purpose.get(tour_purpose)
+    else:
+        raise RuntimeError(f"expected string or dict DESTINATION_FOR_TOUR_PURPOSE model_setting for {tour_purpose}")
 
     odt_skim_stack_wrapper = skim_dict.wrap_3d(orig_key=orig_col_name, dest_key=dest_col_name,
                                                dim3_key='out_period')
@@ -90,8 +99,7 @@ def _compute_logsums(alt_tdd, tours_merged, tour_purpose, model_settings, networ
     logsum_settings = config.read_model_settings(model_settings['LOGSUM_SETTINGS'])
 
     choosers = alt_tdd.join(tours_merged, how='left', rsuffix='_chooser')
-    logger.info("%s compute_logsums for %d choosers%s alts" %
-                (trace_label, choosers.shape[0], alt_tdd.shape[0]))
+    logger.info(f"{trace_label} compute_logsums for {choosers.shape[0]} choosers {alt_tdd.shape[0]} alts")
 
     # - locals_dict
     constants = config.get_model_constants(logsum_settings)
@@ -128,7 +136,7 @@ def _compute_logsums(alt_tdd, tours_merged, tour_purpose, model_settings, networ
     logsum_spec = simulate.eval_coefficients(logsum_spec, coefficients, estimator=None)
 
     nest_spec = config.get_logit_model_settings(logsum_settings)
-    nest_spec = simulate.eval_nest_coefficients(nest_spec, coefficients)
+    nest_spec = simulate.eval_nest_coefficients(nest_spec, coefficients, trace_label)
 
     logsums = simulate.simple_simulate_logsums(
         choosers,
@@ -142,6 +150,82 @@ def _compute_logsums(alt_tdd, tours_merged, tour_purpose, model_settings, networ
     return logsums
 
 
+def dedupe_alt_tdd(alt_tdd, tour_purpose, trace_label):
+
+    tdd_segments = inject.get_injectable('tdd_alt_segments', None)
+    alt_tdd_periods = None
+
+    if tdd_segments is not None:
+
+        dedupe_columns = ['out_period', 'in_period']
+
+        # tdd_alt_segments is optionally segmented by tour purpose
+        if 'tour_purpose' in tdd_segments:
+
+            is_tdd_for_tour_purpose = (tdd_segments.tour_purpose == tour_purpose)
+            if not is_tdd_for_tour_purpose.any():
+                is_tdd_for_tour_purpose = tdd_segments.tour_purpose.isnull()
+            assert is_tdd_for_tour_purpose.any(), \
+                f"no segments found for tour purpose {tour_purpose} in tour_departure_and_duration_segments"
+
+            tdd_segments = tdd_segments[is_tdd_for_tour_purpose].drop(columns=['tour_purpose'])
+            assert len(tdd_segments) > 0, f"tour_purpose '{tour_purpose}' not in tdd_alt_segments"
+
+        # left join representative start on out_period
+        alt_tdd_periods = \
+            pd.merge(alt_tdd[['out_period', 'in_period']].reset_index(),
+                     tdd_segments[['time_period', 'start']].rename(columns={'time_period': 'out_period'}),
+                     how='left', on='out_period')
+
+        # left join representative end on in_period
+        alt_tdd_periods = \
+            pd.merge(alt_tdd_periods,
+                     tdd_segments[['time_period', 'end']].rename(columns={'time_period': 'in_period'}),
+                     how='left', on=['in_period'])
+
+        if tdd_segments.start.isnull().any():
+            missing_periods = tdd_segments.out_period[tdd_segments.start.isnull()].unique()
+            logger.warning(f"missing out_periods in tdd_alt_segments: {missing_periods}")
+
+        if tdd_segments.end.isnull().any():
+            missing_periods = tdd_segments.in_period[tdd_segments.end.isnull()].unique()
+            logger.warning(f"missing in_periods in tdd_alt_segments: {missing_periods}")
+
+        assert not tdd_segments.start.isnull().any()
+        assert not tdd_segments.end.isnull().any()
+
+        # drop duplicates
+        alt_tdd_periods = alt_tdd_periods.drop_duplicates().set_index(alt_tdd.index.name)
+
+        # representative duration
+        alt_tdd_periods['duration'] = alt_tdd_periods['end'] - alt_tdd_periods['start']
+
+        logger.debug(f"{trace_label} "
+                     f"dedupe_alt_tdd.tdd_alt_segments reduced number of rows by "
+                     f"{round(100 * (len(alt_tdd) - len(alt_tdd_periods)) / len(alt_tdd), 2)}% "
+                     f"from {len(alt_tdd)} to {len(alt_tdd_periods)}")
+
+    # if there is no tdd_alt_segments file, we can at least dedupe on 'out_period', 'in_period', 'duration'
+    if alt_tdd_periods is None:
+
+        # FIXME This won't work if they reference start or end in logsum calculations
+        # for MTC only duration is used (to calculate all_day parking cost)
+        dedupe_columns = ['out_period', 'in_period', 'duration']
+
+        logger.warning(f"No tdd_alt_segments so fallback to deduping tdd_alts by time_period and duration")
+
+        # - get list of unique (tour_id, out_period, in_period, duration) in alt_tdd_periods
+        # we can cut the number of alts roughly in half (for mtctm1) by conflating duplicates
+        alt_tdd_periods = alt_tdd[dedupe_columns].reset_index().drop_duplicates().set_index(alt_tdd.index.name)
+
+        logger.debug(f"{trace_label} "
+                     f"dedupe_alt_tdd.drop_duplicates reduced number of rows by "
+                     f"{round(100 * (len(alt_tdd) - len(alt_tdd_periods)) / len(alt_tdd), 2)}% "
+                     f"from {len(alt_tdd)} to {len(alt_tdd_periods)}")
+
+    return alt_tdd_periods, dedupe_columns
+
+
 def compute_logsums(alt_tdd, tours_merged, tour_purpose, model_settings, skims, trace_label):
     """
     Compute logsums for the tour alt_tdds, which will differ based on their different start, stop
@@ -155,6 +239,7 @@ def compute_logsums(alt_tdd, tours_merged, tour_purpose, model_settings, skims,
     (out-period, in-period) pairs and then join them back to the alt_tdds.
     """
 
+    trace_label = tracing.extend_trace_label(trace_label, 'compute_logsums')
     network_los = inject.get_injectable('network_los')
 
     # - in_period and out_period
@@ -169,28 +254,40 @@ def compute_logsums(alt_tdd, tours_merged, tour_purpose, model_settings, skims,
         logsums = _compute_logsums(alt_tdd, tours_merged, tour_purpose, model_settings, network_los, skims, trace_label)
         return logsums
 
-    # - get list of unique (tour_id, out_period, in_period, duration) in alt_tdd_periods
-    # we can cut the number of alts roughly in half (for mtctm1) by conflating duplicates
     index_name = alt_tdd.index.name
-    alt_tdd_periods = alt_tdd[['out_period', 'in_period', 'duration']]\
-        .reset_index().drop_duplicates().set_index(index_name)
+    deduped_alt_tdds, redupe_columns = dedupe_alt_tdd(alt_tdd, tour_purpose, trace_label)
+
+    logger.info(f"{trace_label} compute_logsums "
+                f"deduped_alt_tdds reduced number of rows by "
+                f"{round(100 * (len(alt_tdd) - len(deduped_alt_tdds)) / len(alt_tdd), 2)}% "
+                f"from {len(alt_tdd)} to {len(deduped_alt_tdds)} compared to USE_BRUTE_FORCE_TO_COMPUTE_LOGSUMS")
+
+    t0 = tracing.print_elapsed_time()
 
     # - compute logsums for the alt_tdd_periods
-    alt_tdd_periods['logsums'] = \
-        _compute_logsums(alt_tdd_periods, tours_merged, tour_purpose, model_settings, network_los, skims, trace_label)
+    deduped_alt_tdds['logsums'] = \
+        _compute_logsums(deduped_alt_tdds, tours_merged, tour_purpose, model_settings, network_los, skims, trace_label)
 
-    logger.debug(f"{trace_label} compute_logsums "
-                 f"alt_tdd_periods reduced number of rows by {round(100*len(alt_tdd_periods)/len(alt_tdd), 2)}% "
-                 f" compared to alt_tdd len when USE_BRUTE_FORCE_TO_COMPUTE_LOGSUMS")
+    # tracing.log_runtime(model_name=trace_label, start_time=t0)
 
-    # - join the alt_tdd_period logsums to alt_tdd to get logsums for alt_tdd
+    # redupe - join the alt_tdd_period logsums to alt_tdd to get logsums for alt_tdd
     logsums = pd.merge(
         alt_tdd.reset_index(),
-        alt_tdd_periods.reset_index(),
-        on=[index_name, 'out_period', 'in_period', 'duration'],
+        deduped_alt_tdds.reset_index(),
+        on=[index_name] + redupe_columns,
         how='left'
     ).set_index(index_name).logsums
 
+    # this is really expensive
+    TRACE = False
+    if TRACE:
+        trace_logsums_df = logsums.to_frame('representative_logsum')
+        trace_logsums_df['brute_force_logsum'] = \
+            _compute_logsums(alt_tdd, tours_merged, tour_purpose, model_settings, network_los, skims, trace_label)
+        tracing.trace_df(trace_logsums_df,
+                         label=tracing.extend_trace_label(trace_label, 'representative_logsums'),
+                         slicer='NONE', transpose=False)
+
     return logsums
 
 
@@ -288,6 +385,57 @@ def tdd_interaction_dataset(tours, alts, timetable, choice_column, window_id_col
     return alt_tdd
 
 
+def run_alts_preprocessor(model_settings, alts, segment, locals_dict, trace_label):
+    """
+    run preprocessor on alts, as specified by ALTS_PREPROCESSOR in model_settings
+
+    we are agnostic on whether alts are merged or not
+
+    Parameters
+    ----------
+    model_settings: dict
+        yaml model settings file as dict
+    alts: pandas.DataFrame
+        tdd_alts or tdd_alts merged wiht choosers (we are agnostic)
+    segment: string
+        segment selector as understood by caller (e.g. logsum_tour_purpose)
+    locals_dict: dict
+        we let caller worry about what needs to be in it. though actually depends on modelers needs
+    trace_label: string
+
+    Returns
+    -------
+    alts: pandas.DataFrame
+        annotated copy of alts
+    """
+
+    preprocessor_settings = model_settings.get('ALTS_PREPROCESSOR', {})
+
+    if segment in preprocessor_settings:
+        # segmented by logsum_tour_purpose
+        preprocessor_settings = preprocessor_settings.get(segment)
+        logger.debug(f"running ALTS_PREPROCESSOR with spec for {segment}: {preprocessor_settings.get('SPEC')}")
+    elif 'SPEC' in preprocessor_settings:
+        # unsegmented (either because no segmentation, or fallback if settings has generic preprocessor)
+        logger.debug(f"running ALTS_PREPROCESSOR with unsegmented spec {preprocessor_settings.get('SPEC')}")
+    else:
+        logger.debug(f"skipping alts preprocesser because no ALTS_PREPROCESSOR segment for {segment}")
+        preprocessor_settings = None
+
+    if preprocessor_settings:
+
+        logger.debug(f"run_alts_preprocessor calling assign_columns for {segment} preprocessor_settings")
+        alts = alts.copy()
+
+        expressions.assign_columns(
+            df=alts,
+            model_settings=preprocessor_settings,
+            locals_dict=locals_dict,
+            trace_label=trace_label)
+
+    return alts
+
+
 def _schedule_tours(
         tours, persons_merged, alts,
         spec, logsum_tour_purpose,
@@ -364,7 +512,7 @@ def _schedule_tours(
     choice_column = TDD_CHOICE_COLUMN
     alt_tdd = tdd_interaction_dataset(tours, alts, timetable, choice_column, window_id_col,
                                       tour_trace_label)
-    print(f"tours {tours.shape} alts {alts.shape}")
+    # print(f"tours {tours.shape} alts {alts.shape}")
     chunk.log_df(tour_trace_label, "alt_tdd", alt_tdd)
 
     # - add logsums
@@ -389,14 +537,17 @@ def _schedule_tours(
     if constants is not None:
         locals_d.update(constants)
 
-    preprocessor_settings = model_settings.get('ALTS_PREPROCESSOR', None)
-
-    if preprocessor_settings and preprocessor_settings.get(logsum_tour_purpose):
-        expressions.assign_columns(
-            df=alt_tdd,
-            model_settings=preprocessor_settings.get(logsum_tour_purpose),
-            locals_dict=locals_d,
-            trace_label=tour_trace_label)
+    if not RUN_ALTS_PREPROCESSOR_BEFORE_MERGE:
+        # Note: Clint was running alts_preprocessor here on tdd_interaction_dataset instead of on raw (unmerged) alts
+        # and he was using logsum_tour_purpose as selector, although logically it should be the spec_segment
+        # It just happened to work for example_arc.mandatory_tour_scheduling because, in that model, (unlike semcog)
+        # logsum_tour_purpose and spec_segments are aligned (both logsums and spec are segmented on work, school, univ)
+        # In any case, I don't see any benefit to doing this here - at least not for any existing implementations
+        # but if we do, it will require passing spec_segment to schedule_tours  and _schedule_tours
+        # or redundently segmenting alts (yuck!) to conform to more granular tour_segmentation (e.g. univ do school)
+        spec_segment = logsum_tour_purpose  # FIXME this is not always right - see note above
+        alt_tdd = run_alts_preprocessor(model_settings, alt_tdd, spec_segment, locals_d, tour_trace_label)
+        chunk.log_df(tour_trace_label, "alt_tdd", alt_tdd)
 
     if estimator:
         # write choosers after annotation
@@ -428,7 +579,7 @@ def _schedule_tours(
 
 def tour_scheduling_calc_row_size(tours, persons_merged, alternatives, skims, spec, model_settings, trace_label):
 
-    # this will no be consistent across mandatory tours (highest), non_mandatory tours, and atwork subtours (lowest)
+    # this will not be consistent across mandatory tours (highest), non_mandatory tours, and atwork subtours (lowest)
     TIMETABLE_AVAILABILITY_REDUCTION_FACTOR = 1
     # this appears to be more stable
     LOGSUM_DUPLICATE_REDUCTION_FACTOR = 0.5
@@ -643,15 +794,20 @@ def vectorize_tour_scheduling(tours, persons_merged, alts, timetable,
 
                 segment_trace_label = tracing.extend_trace_label(tour_trace_label, tour_segment_name)
 
-                # assume segmentation of spec and logsum coefficients are aligned
+                # assume segmentation of spec and coefficients are aligned
                 spec_segment_name = tour_segment_info.get('spec_segment_name')
-                logsum_tour_purpose = spec_segment_name if compute_logsums else None
+                # assume logsum segmentation is same as tours
+                logsum_tour_purpose = tour_segment_name if compute_logsums else None
 
                 nth_tours_in_segment = nth_tours[nth_tours[tour_segment_col] == tour_segment_name]
                 if nth_tours_in_segment.empty:
                     logger.info("skipping empty segment %s" % tour_segment_name)
                     continue
 
+                if RUN_ALTS_PREPROCESSOR_BEFORE_MERGE:
+                    locals_dict = {}
+                    alts = run_alts_preprocessor(model_settings, alts, spec_segment_name, locals_dict, tour_trace_label)
+
                 choices = \
                     schedule_tours(nth_tours_in_segment, persons_merged, alts,
                                    spec=tour_segment_info['spec'],
@@ -669,6 +825,9 @@ def vectorize_tour_scheduling(tours, persons_merged, alts, timetable,
 
         else:
 
+            # MTC non_mandatory_tours are not segmented by tour_purpose and do not require logsums
+            # FIXME should support logsums?
+
             assert not compute_logsums, "logsums for unsegmented spec not implemented because not currently needed"
             assert tour_segments.get('spec_segment_name') is None
 
@@ -701,7 +860,7 @@ def vectorize_subtour_scheduling(parent_tours, subtours, persons_merged, alts, s
     subtours have a few peculiarities necessitating separate treatment:
 
     Timetable has to be initialized to set all timeperiods outside parent tour footprint as
-    unavailable. So atwork subtour timewindows are limited to the foorprint of the parent work
+    unavailable. So atwork subtour timewindows are limited to the footprint of the parent work
     tour. And parent_tour_id' column of tours is used instead of parent_id as timetable row_id.
 
     Parameters
@@ -738,7 +897,7 @@ def vectorize_subtour_scheduling(parent_tours, subtours, persons_merged, alts, s
 
     timetable_window_id_col = 'parent_tour_id'
     tour_owner_id_col = 'parent_tour_id'
-    segment = None
+    logsum_tour_purpose = None  # FIXME logsums not currently supported
 
     # timetable with a window for each parent tour
     parent_tour_windows = tt.create_timetable_windows(parent_tours, alts)
@@ -776,7 +935,7 @@ def vectorize_subtour_scheduling(parent_tours, subtours, persons_merged, alts, s
         choices = \
             schedule_tours(nth_tours,
                            persons_merged, alts,
-                           spec, segment,
+                           spec, logsum_tour_purpose,
                            model_settings,
                            timetable, timetable_window_id_col,
                            previous_tour_by_parent_tour_id, tour_owner_id_col,
@@ -863,7 +1022,7 @@ def vectorize_joint_tour_scheduling(
 
     timetable_window_id_col = None
     tour_owner_id_col = 'household_id'
-    segment = None
+    logsum_tour_purpose = None  # FIXME logsums not currently supported
 
     choice_list = []
 
@@ -895,7 +1054,8 @@ def vectorize_joint_tour_scheduling(
         choices = \
             schedule_tours(nth_tours,
                            persons_merged, alts,
-                           spec, segment,
+                           spec,
+                           logsum_tour_purpose,
                            model_settings,
                            timetable, timetable_window_id_col,
                            previous_tour_by_householdid, tour_owner_id_col,
