@@ -4,6 +4,7 @@ import sys
 import os
 import logging
 import argparse
+import warnings
 
 from activitysim.core import inject
 from activitysim.core import tracing
@@ -75,8 +76,6 @@ def validate_injectable(name):
         if not os.path.exists(dir_path):
             sys.exit("Could not find %s '%s'" % (name, os.path.abspath(dir_path)))
 
-    logger.info('using %s: %s' % (name, dir_paths))
-
     return dir_paths
 
 
@@ -127,9 +126,7 @@ def handle_standard_args(args, multiprocess=True):
 
 def cleanup_output_files():
 
-    active_log_files = \
-        [h.baseFilename for h in logger.root.handlers if isinstance(h, logging.FileHandler)]
-    tracing.delete_output_files('log', ignore=active_log_files)
+    tracing.delete_trace_files()
 
     tracing.delete_output_files('h5')
     tracing.delete_output_files('csv')
@@ -139,20 +136,6 @@ def cleanup_output_files():
     tracing.delete_output_files('omx')
 
 
-def log_settings():
-
-    settings = [
-        'households_sample_size',
-        'chunk_size',
-        'multiprocess',
-        'num_processes',
-        'resume_after',
-    ]
-
-    for k in settings:
-        logger.info('setting %s: %s' % (k, config.setting(k)))
-
-
 def run(args):
     """
     Run the models. Specify a project folder using the '--working_dir' option,
@@ -160,19 +143,36 @@ def run(args):
     '--config', '--data', and '--output'. Both '--config' and '--data' can be
     specified multiple times. Directories listed first take precedence.
 
+    returns:
+        int: sys.exit exit code
     """
 
-    from activitysim import abm  # register injectables
+    # register abm steps and other abm-specific injectables
+    # by default, assume we are running activitysim.abm
+    # other callers (e.g. populationsim) will have to arrange to register their own steps and injectables
+    # (presumably) in a custom run_simulation.py instead of using the 'activitysim run' command
+    if not inject.is_injectable('preload_injectables'):
+        from activitysim import abm  # register abm steps and other abm-specific injectables
 
     tracing.config_logger(basic=True)
     handle_standard_args(args)  # possibly update injectables
-    tracing.config_logger(basic=False)  # update using possibly new logging configs
-    config.filter_warnings()
-    logging.captureWarnings(capture=True)
 
-    log_settings()
+    # legacy support for run_list setting nested 'models' and 'resume_after' settings
+    if config.setting('run_list'):
+        warnings.warn("Support for 'run_list' settings group will be removed.\n"
+                      "The run_list.steps setting is renamed 'models'.\n"
+                      "The run_list.resume_after setting is renamed 'resume_after'.\n"
+                      "Specify both 'models' and 'resume_after' directly in settings config file.", FutureWarning)
+        run_list = config.setting('run_list')
+        if 'steps' in run_list:
+            assert not config.setting('models'), \
+                f"Don't expect 'steps' in run_list and 'models' as stand-alone setting!"
+            config.override_setting('models', run_list['steps'])
 
-    t0 = tracing.print_elapsed_time()
+        if 'resume_after' in run_list:
+            assert not config.setting('resume_after'), \
+                f"Don't expect 'resume_after' both in run_list and as stand-alone setting!"
+            config.override_setting('resume_after', run_list['resume_after'])
 
     # If you provide a resume_after argument to pipeline.run
     # the pipeline manager will attempt to load checkpointed tables from the checkpoint store
@@ -185,6 +185,20 @@ def run(args):
     elif config.setting('cleanup_trace_files_on_resume', False):
         tracing.delete_trace_files()
 
+    tracing.config_logger(basic=False)  # update using possibly new logging configs
+    config.filter_warnings()
+    logging.captureWarnings(capture=True)
+
+    # directories
+    for k in ['configs_dir', 'settings_file_name', 'data_dir', 'output_dir']:
+        logger.info('SETTING %s: %s' % (k, inject.get_injectable(k, None)))
+
+    log_settings = inject.get_injectable('log_settings', {})
+    for k in log_settings:
+        logger.info('SETTING %s: %s' % (k, config.setting(k)))
+
+    t0 = tracing.print_elapsed_time()
+
     if config.setting('multiprocess', False):
         logger.info('run multiprocess simulation')
 
@@ -192,18 +206,36 @@ def run(args):
         run_list = mp_tasks.get_run_list()
         injectables = {k: inject.get_injectable(k) for k in INJECTABLES}
         mp_tasks.run_multiprocess(run_list, injectables)
+
+        assert not pipeline.is_open()
+
+        if config.setting('cleanup_pipeline_after_run', False):
+            pipeline.cleanup_pipeline()
+
     else:
         logger.info('run single process simulation')
 
         pipeline.run(models=config.setting('models'), resume_after=resume_after)
-        pipeline.close_pipeline()
+
+        if config.setting('cleanup_pipeline_after_run', False):
+            pipeline.cleanup_pipeline()  # has side effect of closing open pipeline
+        else:
+            pipeline.close_pipeline()
+
         chunk.log_write_hwm()
 
     tracing.print_elapsed_time('all models', t0)
 
+    return 0
+
 
 if __name__ == '__main__':
+
+    from activitysim import abm  # register injectables
+
     parser = argparse.ArgumentParser()
     add_run_args(parser)
     args = parser.parse_args()
+
+    parser.parse_args(['--sum', '7', '-1', '42'])
     sys.exit(run(args))
