@@ -3,7 +3,7 @@
 from builtins import range
 
 import logging
-
+import warnings
 import numpy as np
 import pandas as pd
 
@@ -213,7 +213,7 @@ class TransitVirtualPathBuilder(object):
                                   trace_label, trace):
         """
         create transit_df and compute utilities for all atap-btap pairs between omaz in access and dmaz in egress_df
-        compute the utilities using the tap_tap utilitiy expressions file specified in tap_tap_settings
+        compute the utilities using the tap_tap utility expressions file specified in tap_tap_settings
 
         transit_df contains all possible access omaz/btap to egress dmaz/atap transit path pairs for each chooser
 
@@ -398,7 +398,7 @@ class TransitVirtualPathBuilder(object):
 
         return transit_df
 
-    def compute_tap_tap_time(self, recipe, access_df, egress_df, chooser_attributes, trace_label, trace):
+    def compute_tap_tap_time(self, recipe, access_df, egress_df, chooser_attributes, path_info, trace_label, trace):
 
         trace_label = tracing.extend_trace_label(trace_label, 'compute_tap_tap_time')
 
@@ -410,14 +410,47 @@ class TransitVirtualPathBuilder(object):
             # note: transit_df index is arbitrary
             chunk.log_df(trace_label, "transit_df", transit_df)
 
-        locals_d = {'los': self.network_los}
-        locals_d.update(model_constants)
+        # some expressions may want to know access mode -
+        locals_dict = path_info.copy()
+        locals_dict['los'] = self.network_los
+        locals_dict.update(model_constants)
 
         assignment_spec = assign.read_assignment_spec(file_name=config.config_file_path(tap_tap_settings['SPEC']))
 
-        results, _, _ = assign.assign_variables(assignment_spec, transit_df, locals_d)
-        assert len(results.columns == 1)
-        transit_df['transit'] = results
+        DEDUPE = True
+        if DEDUPE:
+
+            # assign uid for reduping
+            max_atap = transit_df.atap.max() + 1
+            transit_df['uid'] = transit_df.btap * max_atap + transit_df.atap
+
+            # dedupe
+            chooser_attribute_columns = list(chooser_attributes.columns)
+            unique_transit_df = \
+                transit_df.loc[~transit_df.uid.duplicated(), ['btap', 'atap', 'uid'] + chooser_attribute_columns]
+            unique_transit_df.set_index('uid', inplace=True)
+            chunk.log_df(trace_label, "unique_transit_df", unique_transit_df)
+
+            logger.debug(f"#TVPB CACHE deduped transit_df from {len(transit_df)} to {len(unique_transit_df)}")
+
+            # assign_variables
+            results, _, _ = assign.assign_variables(assignment_spec, unique_transit_df, locals_dict)
+            assert len(results.columns == 1)
+            unique_transit_df['transit'] = results
+
+            # redupe results back into transit_df
+            with memo("#TVPB compute_tap_tap_time redupe transit_df"):
+                transit_df['transit'] = reindex(unique_transit_df.transit, transit_df.uid)
+
+            del transit_df['uid']
+            del unique_transit_df
+            chunk.log_df(trace_label, "transit_df", transit_df)
+            chunk.log_df(trace_label, "unique_transit_df", None)
+
+        else:
+            results, _, _ = assign.assign_variables(assignment_spec, transit_df, locals_dict)
+            assert len(results.columns == 1)
+            transit_df['transit'] = results
 
         # filter out unavailable btap_atap pairs
         logger.debug(f"{(transit_df['transit'] <= 0).sum()} unavailable tap_tap pairs out of {len(transit_df)}")
@@ -452,8 +485,10 @@ class TransitVirtualPathBuilder(object):
             return result
         else:
             assert self.units_for_recipe(recipe) == 'time'
-            result = self.compute_tap_tap_time(recipe, access_df, egress_df, chooser_attributes, trace_label, trace)
 
+            with memo("#TVPB compute_tap_tap_time"):
+                result = self.compute_tap_tap_time(recipe, access_df, egress_df, chooser_attributes,
+                                                   path_info, trace_label, trace)
         return result
 
     def best_paths(self, recipe, path_type, maz_od_df, access_df, egress_df, transit_df, trace_label, trace=False):
@@ -553,7 +588,10 @@ class TransitVirtualPathBuilder(object):
         access_mode = self.network_los.setting(f'TVPB_SETTINGS.{recipe}.path_types.{path_type}.access')
         egress_mode = self.network_los.setting(f'TVPB_SETTINGS.{recipe}.path_types.{path_type}.egress')
         path_types_settings = self.network_los.setting(f'TVPB_SETTINGS.{recipe}.path_types.{path_type}')
-        paths_nest_nesting_coefficient = path_types_settings.get('paths_nest_nesting_coefficient', 1)
+        attributes_as_columns = \
+            self.network_los.setting(f'TVPB_SETTINGS.{recipe}.tap_tap_settings.attributes_as_columns', [])
+
+        path_info = {'path_type': path_type, 'access_mode': access_mode, 'egress_mode': egress_mode}
 
         # maz od pairs requested
         with memo("#TVPB build_virtual_path maz_od_df"):
@@ -571,7 +609,12 @@ class TransitVirtualPathBuilder(object):
         # knowing this allows us to eliminate redundant computations (e.g. utilities of maz_tap pairs)
         duplicated = orig.index.duplicated(keep='first')
         chooser_attributes = pd.DataFrame(index=orig.index[~duplicated])
-        chooser_attributes['tod'] = tod if isinstance(tod, str) else tod.loc[~duplicated]
+        if not isinstance(tod, str):
+            chooser_attributes['tod'] = tod.loc[~duplicated]
+        elif 'tod' in attributes_as_columns:
+            chooser_attributes['tod'] = tod
+        else:
+            path_info['tod'] = tod
         if demographic_segment is not None:
             chooser_attributes['demographic_segment'] = demographic_segment.loc[~duplicated]
 
@@ -600,7 +643,6 @@ class TransitVirtualPathBuilder(object):
         # path_info for use by expressions (e.g. penalty for drive access if no parking at access tap)
         with memo("#TVPB build_virtual_path compute_tap_tap"):
             with chunk.chunk_log(f'#TVPB.compute_tap_tap'):
-                path_info = {'path_type': path_type, 'access_mode': access_mode, 'egress_mode': egress_mode}
                 transit_df = self.compute_tap_tap(
                     recipe,
                     maz_od_df,
@@ -645,8 +687,25 @@ class TransitVirtualPathBuilder(object):
 
                 chunk.log_df(trace_label, "utilities_df", utilities_df)
 
-                logsums = np.maximum(np.log(np.nansum(np.exp(utilities_df.values/paths_nest_nesting_coefficient),
-                                                      axis=1)), UNAVAILABLE)
+                with warnings.catch_warnings(record=True) as w:
+                    # Cause all warnings to always be triggered.
+                    # most likely "divide by zero encountered in log" caused by all transit sets non-viable
+                    warnings.simplefilter("always")
+
+                    paths_nest_nesting_coefficient = path_types_settings.get('paths_nest_nesting_coefficient', 1)
+                    exp_utilities = np.exp(utilities_df.values / paths_nest_nesting_coefficient)
+                    logsums = np.maximum(np.log(np.nansum(exp_utilities, axis=1)), UNAVAILABLE)
+
+                    if len(w) > 0:
+                        for wrn in w:
+                            logger.warning(
+                                f"{trace_label} - {type(wrn).__name__} ({wrn.message})")
+
+                        DUMP = False
+                        if DUMP:
+                            zero_utilities_df = utilities_df[np.nansum(np.exp(utilities_df.values), axis=1) == 0]
+                            zero_utilities_df.to_csv(config.output_file_path('warning_utilities_df.csv'), index=True)
+                            bug
 
             if want_choices:
 
